@@ -45,6 +45,9 @@ WebElement <- R6::R6Class("WebElement",
     #' tree. A shadow root is an element that contains a DOM subtree. This
     #' method gets the shadow root property of an element.
     #'
+    #' @param timeout How long to wait for a request to recieve a response
+    #'   before throwing an error.
+    #'
     #' @return A [ShadowRoot] object.
     #'
     #' @examples
@@ -64,9 +67,11 @@ WebElement <- R6::R6Class("WebElement",
     #'
     #' session$close()
     #' }
-    shadow_root = function() {
+    shadow_root = function(timeout = 20) {
+      check_number_decimal(timeout, allow_null = TRUE)
+
       req <- req_command(private$req, "Get Element Shadow Root", session_id = private$session_id, element_id = self$id)
-      response <- req_perform_selenium(req, verbose = private$verbose)
+      response <- req_perform_selenium(req, verbose = private$verbose, timeout = timeout)
       id <- httr2::resp_body_json(response)$value[[1]]
       ShadowRoot$new(private$session_id, private$req, private$verbose, id)
     },
@@ -78,6 +83,8 @@ WebElement <- R6::R6Class("WebElement",
     #' @param value The value of the selector: a string.
     #' @param request_body A list of request body parameters to pass to the
     #'   Selenium server, overriding the default body of the web request
+    #' @param timeout How long to wait for a request to recieve a response
+    #'   before throwing an error.
     #'
     #' @return A `WebElement` object.
     #'
@@ -97,14 +104,16 @@ WebElement <- R6::R6Class("WebElement",
     #' }
     find_element = function(using = c("css selector", "xpath", "tag name", "link text", "partial link text"),
                             value,
-                            request_body = NULL) {
+                            request_body = NULL,
+                            timeout = 20) {
       using <- rlang::arg_match(using)
       check_string(value)
       check_list(request_body, allow_null = TRUE)
+      check_number_decimal(timeout, allow_null = TRUE)
 
       req <- req_command(private$req, "Find Element From Element", session_id = private$session_id, element_id = self$id)
       req <- req_body_selenium(req, list(using = using, value = value), request_body = request_body)
-      response <- req_perform_selenium(req, verbose = private$verbose)
+      response <- req_perform_selenium(req, verbose = private$verbose, timeout = timeout)
       id <- httr2::resp_body_json(response)$value[[1]]
       WebElement$new(private$session_id, private$req, private$verbose, id)
     },
@@ -115,6 +124,8 @@ WebElement <- R6::R6Class("WebElement",
     #' @param value The value of the selector: a string.
     #' @param request_body A list of request body parameters to pass to the
     #'   Selenium server, overriding the default body of the web request
+    #' @param timeout How long to wait for a request to recieve a response
+    #'   before throwing an error.
     #'
     #' @return A list of `WebElement` objects.
     #'
@@ -132,20 +143,24 @@ WebElement <- R6::R6Class("WebElement",
     #' }
     find_elements = function(using = c("css selector", "xpath", "tag name", "link text", "partial link text"),
                              value,
-                             request_body = NULL) {
+                             request_body = NULL, timeout = 20) {
       using <- rlang::arg_match(using)
       check_string(value)
       check_list(request_body, allow_null = TRUE)
+      check_number_decimal(timeout, allow_null = TRUE)
 
       req <- req_command(private$req, "Find Elements From Element", session_id = private$session_id, element_id = self$id)
       req <- req_body_selenium(req, list(using = using, value = value), request_body = request_body)
-      response <- req_perform_selenium(req, verbose = private$verbose)
+      response <- req_perform_selenium(req, verbose = private$verbose, timeout = timeout)
       ids <- httr2::resp_body_json(response)$value
       lapply(ids, function(id) WebElement$new(private$session_id, private$req, private$verbose, id[[1]]))
     },
     #' @description
     #' Check if an element is currently selected.
     #'
+    #' @param timeout How long to wait for a request to recieve a response
+    #'   before throwing an error.
+    #'
     #' @return A boolean value: `TRUE` or `FALSE`.
     #'
     #' @examples
@@ -158,9 +173,11 @@ WebElement <- R6::R6Class("WebElement",
     #'
     #' session$close()
     #' }
-    is_selected = function() {
+    is_selected = function(timeout = 20) {
+      check_number_decimal(timeout, allow_null = TRUE)
+
       req <- req_command(private$req, "Is Element Selected", session_id = private$session_id, element_id = self$id)
-      response <- req_perform_selenium(req, verbose = private$verbose)
+      response <- req_perform_selenium(req, verbose = private$verbose, timeout = timeout)
       httr2::resp_body_json(response)$value
     },
     #' @description
@@ -169,6 +186,8 @@ WebElement <- R6::R6Class("WebElement",
     #' @param name The name of the attribute.
     #' @param request_body A list of request body parameters to pass to the
     #'   Selenium server, overriding the default body of the web request
+    #' @param timeout How long to wait for a request to recieve a response
+    #'   before throwing an error.
     #'
     #' @return The value of the attribute: a string.
     #'
@@ -182,12 +201,13 @@ WebElement <- R6::R6Class("WebElement",
     #'
     #' session$close()
     #' }
-    get_attribute = function(name, request_body = NULL) {
+    get_attribute = function(name, request_body = NULL, timeout = 20) {
       check_string(name)
       check_list(request_body, allow_null = TRUE)
+      check_number_decimal(timeout, allow_null = TRUE)
 
       req <- req_command(private$req, "Get Element Attribute", session_id = private$session_id, element_id = self$id, name = name)
-      response <- req_perform_selenium(req, verbose = private$verbose)
+      response <- req_perform_selenium(req, verbose = private$verbose, timeout = timeout)
       httr2::resp_body_json(response)$value
     },
     #' @description
@@ -198,6 +218,8 @@ WebElement <- R6::R6Class("WebElement",
     #' @param name The name of the property.
     #' @param request_body A list of request body parameters to pass to the
     #'   Selenium server, overriding the default body of the web request
+    #' @param timeout How long to wait for a request to recieve a response
+    #'   before throwing an error.
     #'
     #' @return The value of the property: a string.
     #'
@@ -211,12 +233,13 @@ WebElement <- R6::R6Class("WebElement",
     #'
     #' session$close()
     #' }
-    get_property = function(name, request_body = NULL) {
+    get_property = function(name, request_body = NULL, timeout = 20) {
       check_string(name)
       check_list(request_body, allow_null = TRUE)
+      check_number_decimal(timeout, allow_null = TRUE)
 
       req <- req_command(private$req, "Get Element Property", session_id = private$session_id, element_id = self$id, name = name)
-      response <- req_perform_selenium(req, verbose = private$verbose)
+      response <- req_perform_selenium(req, verbose = private$verbose, timeout = timeout)
       httr2::resp_body_json(response)$value
     },
     #' @description
@@ -225,6 +248,8 @@ WebElement <- R6::R6Class("WebElement",
     #' @param name The name of the CSS property.
     #' @param request_body A list of request body parameters to pass to the
     #'   Selenium server, overriding the default body of the web request
+    #' @param timeout How long to wait for a request to recieve a response
+    #'   before throwing an error.
     #'
     #' @return The value of the CSS property: a string.
     #'
@@ -238,17 +263,21 @@ WebElement <- R6::R6Class("WebElement",
     #'
     #' session$close()
     #' }
-    get_css_value = function(name, request_body = NULL) {
+    get_css_value = function(name, request_body = NULL, timeout = 20) {
       check_string(name)
       check_list(request_body, allow_null = TRUE)
+      check_number_decimal(timeout, allow_null = TRUE)
 
       req <- req_command(private$req, "Get Element CSS Value", session_id = private$session_id, element_id = self$id, "property name" = name)
-      response <- req_perform_selenium(req, verbose = private$verbose)
+      response <- req_perform_selenium(req, verbose = private$verbose, timeout = timeout)
       httr2::resp_body_json(response)$value
     },
     #' @description
     #' Get the text content of an element.
     #'
+    #' @param timeout How long to wait for a request to recieve a response
+    #'   before throwing an error.
+    #'
     #' @return The text content of the element: a string.
     #'
     #' @examples
@@ -261,14 +290,19 @@ WebElement <- R6::R6Class("WebElement",
     #'
     #' session$close()
     #' }
-    get_text = function() {
+    get_text = function(timeout = 20) {
+      check_number_decimal(timeout, allow_null = TRUE)
+
       req <- req_command(private$req, "Get Element Text", session_id = private$session_id, element_id = self$id)
-      response <- req_perform_selenium(req, verbose = private$verbose)
+      response <- req_perform_selenium(req, verbose = private$verbose, timeout = timeout)
       httr2::resp_body_json(response)$value
     },
     #' @description
     #' Get the tag name of an element.
     #'
+    #' @param timeout How long to wait for a request to recieve a response
+    #'   before throwing an error.
+    #'
     #' @return The tag name of the element: a string.
     #'
     #' @examples
@@ -281,14 +315,19 @@ WebElement <- R6::R6Class("WebElement",
     #'
     #' session$close()
     #' }
-    get_tag_name = function() {
+    get_tag_name = function(timeout = 20) {
+      check_number_decimal(timeout, allow_null = TRUE)
+
       req <- req_command(private$req, "Get Element Tag Name", session_id = private$session_id, element_id = self$id)
-      response <- req_perform_selenium(req, verbose = private$verbose)
+      response <- req_perform_selenium(req, verbose = private$verbose, timeout = timeout)
       httr2::resp_body_json(response)$value
     },
     #' @description
     #' Get the dimensions and coordinates of an element.
     #'
+    #' @param timeout How long to wait for a request to recieve a response
+    #'   before throwing an error.
+    #'
     #' @return A list containing the following elements:
     #'
     #' * `x`: The x-coordinate of the element.
@@ -306,14 +345,19 @@ WebElement <- R6::R6Class("WebElement",
     #'
     #' session$close()
     #' }
-    get_rect = function() {
+    get_rect = function(timeout = 20) {
+      check_number_decimal(timeout, allow_null = TRUE)
+
       req <- req_command(private$req, "Get Element Rect", session_id = private$session_id, element_id = self$id)
-      response <- req_perform_selenium(req, verbose = private$verbose)
+      response <- req_perform_selenium(req, verbose = private$verbose, timeout = timeout)
       httr2::resp_body_json(response)$value
     },
     #' @description
     #' Check if an element is currently enabled.
     #'
+    #' @param timeout How long to wait for a request to recieve a response
+    #'   before throwing an error.
+    #'
     #' @return A boolean value: `TRUE` or `FALSE`.
     #'
     #' @examples
@@ -326,9 +370,11 @@ WebElement <- R6::R6Class("WebElement",
     #'
     #' session$close()
     #' }
-    is_enabled = function() {
+    is_enabled = function(timeout = 20) {
+      check_number_decimal(timeout, allow_null = TRUE)
+
       req <- req_command(private$req, "Is Element Enabled", session_id = private$session_id, element_id = self$id)
-      response <- req_perform_selenium(req, verbose = private$verbose)
+      response <- req_perform_selenium(req, verbose = private$verbose, timeout = timeout)
       httr2::resp_body_json(response)$value
     },
     #' @description
@@ -337,6 +383,9 @@ WebElement <- R6::R6Class("WebElement",
     #' purpose. For example, a link that is "button-like" in nature may have
     #' a "button" role.
     #'
+    #' @param timeout How long to wait for a request to recieve a response
+    #'   before throwing an error.
+    #'
     #' @return A string.
     #'
     #' @examples
@@ -349,15 +398,20 @@ WebElement <- R6::R6Class("WebElement",
     #'
     #' session$close()
     #' }
-    computed_role = function() {
+    computed_role = function(timeout = 20) {
+      check_number_decimal(timeout, allow_null = TRUE)
+
       req <- req_command(private$req, "Get Computed Role", session_id = private$session_id, element_id = self$id)
-      response <- req_perform_selenium(req, verbose = private$verbose)
+      response <- req_perform_selenium(req, verbose = private$verbose, timeout = timeout)
       httr2::resp_body_json(response)$value
     },
     #' @description
     #' Get the computed label of an element (i.e. The text of the label element
     #' that points to the current element).
     #'
+    #' @param timeout How long to wait for a request to recieve a response
+    #'   before throwing an error.
+    #'
     #' @return A string.
     #'
     #' @examples
@@ -370,9 +424,11 @@ WebElement <- R6::R6Class("WebElement",
     #'
     #' session$close()
     #' }
-    computed_label = function() {
+    computed_label = function(timeout = 20) {
+      check_number_decimal(timeout, allow_null = TRUE)
+
       req <- req_command(private$req, "Get Computed Label", session_id = private$session_id, element_id = self$id)
-      response <- req_perform_selenium(req, verbose = private$verbose)
+      response <- req_perform_selenium(req, verbose = private$verbose, timeout = timeout)
       httr2::resp_body_json(response)$value
     },
     #' @description
@@ -380,6 +436,9 @@ WebElement <- R6::R6Class("WebElement",
     #'
     #' @return The element, invisibly.
     #'
+    #' @param timeout How long to wait for a request to recieve a response
+    #'   before throwing an error.
+    #'
     #' @examples
     #' \dontrun{
     #' session <- SeleniumSession$new()
@@ -390,15 +449,20 @@ WebElement <- R6::R6Class("WebElement",
     #'
     #' session$close()
     #' }
-    click = function() {
+    click = function(timeout = 20) {
+      check_number_decimal(timeout, allow_null = TRUE)
+
       req <- req_command(private$req, "Element Click", session_id = private$session_id, element_id = self$id)
       req <- req_body_selenium(req, NULL)
-      req_perform_selenium(req, verbose = private$verbose)
+      req_perform_selenium(req, verbose = private$verbose, timeout = timeout)
       invisible(self)
     },
     #' @description
     #' Clear the contents of a text input element.
     #'
+    #' @param timeout How long to wait for a request to recieve a response
+    #'   before throwing an error.
+    #'
     #' @return The element, invisibly.
     #'
     #' @examples
@@ -411,10 +475,12 @@ WebElement <- R6::R6Class("WebElement",
     #'
     #' session$close()
     #' }
-    clear = function() {
+    clear = function(timeout = 20) {
+      check_number_decimal(timeout, allow_null = TRUE)
+
       req <- req_command(private$req, "Element Clear", session_id = private$session_id, element_id = self$id)
       req <- req_body_selenium(req, NULL)
-      req_perform_selenium(req, verbose = private$verbose)
+      req_perform_selenium(req, verbose = private$verbose, timeout = timeout)
       invisible(self)
     },
     #' @description
@@ -424,6 +490,8 @@ WebElement <- R6::R6Class("WebElement",
     #'   use [key_chord()] to send keys combinations.
     #' @param request_body A list of request body parameters to pass to the
     #'   Selenium server, overriding the default body of the web request
+    #' @param timeout How long to wait for a request to recieve a response
+    #'   before throwing an error.
     #'
     #' @return The element, invisibly.
     #'
@@ -445,23 +513,27 @@ WebElement <- R6::R6Class("WebElement",
     #'
     #' session$close()
     #' }
-    send_keys = function(..., request_body = NULL) {
+    send_keys = function(..., request_body = NULL, timeout = 20) {
       for (a in list2(...)) {
         check_string(a, arg = I("Every element in `...`"))
       }
       check_list(request_body, allow_null = TRUE)
+      check_number_decimal(timeout, allow_null = TRUE)
 
       req <- req_command(private$req, "Element Send Keys", session_id = private$session_id, element_id = self$id)
       body <- list(
         text = paste0(...)
       )
       req <- req_body_selenium(req, body, request_body = request_body)
-      req_perform_selenium(req, verbose = private$verbose)
+      req_perform_selenium(req, verbose = private$verbose, timeout = timeout)
       invisible(self)
     },
     #' @description
     #' Take a screenshot of an element.
     #'
+    #' @param timeout How long to wait for a request to recieve a response
+    #'   before throwing an error.
+    #'
     #' @return The base64-encoded PNG screenshot, as a string.
     #'
     #' @examples
@@ -474,15 +546,20 @@ WebElement <- R6::R6Class("WebElement",
     #'
     #' session$close()
     #' }
-    screenshot = function() {
+    screenshot = function(timeout = 20) {
+      check_number_decimal(timeout, allow_null = TRUE)
+
       req <- req_command(private$req, "Take Element Screenshot", session_id = private$session_id, element_id = self$id)
-      response <- req_perform_selenium(req, verbose = private$verbose)
+      response <- req_perform_selenium(req, verbose = private$verbose, timeout = timeout)
       httr2::resp_body_json(response)$value
     },
     #' @description
     #' Check if an element is displayed. This function may not work on all
     #' platforms.
     #'
+    #' @param timeout How long to wait for a request to recieve a response
+    #'   before throwing an error.
+    #'
     #' @return A boolean.
     #'
     #' @examples
@@ -495,9 +572,11 @@ WebElement <- R6::R6Class("WebElement",
     #'
     #' session$close()
     #' }
-    is_displayed = function() {
+    is_displayed = function(timeout = 20) {
+      check_number_decimal(timeout, allow_null = TRUE)
+
       req <- req_command(private$req, "Element Displayed", session_id = private$session_id, element_id = self$id)
-      response <- req_perform_selenium(req, verbose = private$verbose)
+      response <- req_perform_selenium(req, verbose = private$verbose, timeout = timeout)
       httr2::resp_body_json(response)$value
     },
     #' @description
@@ -590,6 +669,8 @@ ShadowRoot <- R6::R6Class("ShadowRoot",
     #' @param value The value of the selector: a string.
     #' @param request_body A list of request body parameters to pass to the
     #'   Selenium server, overriding the default body of the web request
+    #' @param timeout How long to wait for a request to recieve a response
+    #'   before throwing an error.
     #'
     #' @return A [WebElement] object.
     #'
@@ -616,14 +697,15 @@ ShadowRoot <- R6::R6Class("ShadowRoot",
     #' session$close()
     #' }
     find_element = function(using = c("css selector", "xpath", "tag name", "link text", "partial link text"),
-                            value, request_body = NULL) {
+                            value, request_body = NULL, timeout = 20) {
       using <- rlang::arg_match(using)
       check_string(value)
       check_list(request_body, allow_null = TRUE)
+      check_number_decimal(timeout, allow_null = TRUE)
 
       req <- req_command(private$req, "Find Element From Shadow Root", session_id = private$session_id, shadow_id = self$id)
       req <- req_body_selenium(req, list(using = using, value = value), request_body = request_body)
-      response <- req_perform_selenium(req, verbose = private$verbose)
+      response <- req_perform_selenium(req, verbose = private$verbose, timeout = timeout)
       id <- httr2::resp_body_json(response)$value
       WebElement$new(private$session_id, private$req, private$verbose, id[[1]])
     },
@@ -634,6 +716,8 @@ ShadowRoot <- R6::R6Class("ShadowRoot",
     #' @param value The value of the selector: a string.
     #' @param request_body A list of request body parameters to pass to the
     #'   Selenium server, overriding the default body of the web request
+    #' @param timeout How long to wait for a request to recieve a response
+    #'   before throwing an error.
     #'
     #' @return A list of [WebElement] objects.
     #'
@@ -663,15 +747,16 @@ ShadowRoot <- R6::R6Class("ShadowRoot",
     #' session$close()
     #' }
     find_elements = function(using = c("css selector", "xpath", "tag name", "link text", "partial link text"),
-                             value, request_body = NULL) {
+                             value, request_body = NULL, timeout = 20) {
       using <- rlang::arg_match(using)
       check_string(value)
       check_list(request_body, allow_null = TRUE)
+      check_number_decimal(timeout, allow_null = TRUE)
 
       using <- rlang::arg_match(using)
       req <- req_command(private$req, "Find Elements From Shadow Root", session_id = private$session_id, shadow_id = self$id)
       req <- req_body_selenium(req, list(using = using, value = value), request_body = request_body)
-      response <- req_perform_selenium(req, verbose = private$verbose)
+      response <- req_perform_selenium(req, verbose = private$verbose, timeout = timeout)
       ids <- httr2::resp_body_json(response)$value
       lapply(ids, function(id) WebElement$new(private$session_id, private$req, private$verbose, id[[1]]))
     },
